package com.ascendfinalproject.warehouse.config;

import com.ascendfinalproject.warehouse.models.WarehouseOrderRequest;
import com.ascendfinalproject.warehouse.models.WarehouseOrderResponse;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import reactor.kafka.receiver.KafkaReceiver;
import reactor.kafka.receiver.ReceiverOptions;
import reactor.kafka.sender.KafkaSender;
import reactor.kafka.sender.SenderOptions;
import org.springframework.kafka.support.serializer.JsonSerializer;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

@Configuration
public class KafkaConfig {

  @Value("${kafka.producer.bootstrap-servers}")
  private String bootstrapServers;

  @Value("${kafka.producer.acks}")
  private String acks;

  @Value("${kafka.consumer.group-id}")
  private String groupId;


  @Bean
  public Map<String, Object> consumerFactory() {
    Map<String, Object> receiverConfigProps = new HashMap<>();
    receiverConfigProps.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
    receiverConfigProps.put(ConsumerConfig.GROUP_ID_CONFIG, groupId);
    receiverConfigProps.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
    receiverConfigProps.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
    return receiverConfigProps;
  }

  @Bean
  public KafkaReceiver<String, String> kafkaEventReceiver(@Value("${kafka.topic.input}") String topic) {
    ReceiverOptions<String, String> receiverOptions = ReceiverOptions.create(consumerFactory());
    receiverOptions.maxCommitAttempts(5);
    return KafkaReceiver.create(receiverOptions.addAssignListener(Collection::iterator).subscription(Collections.singleton(topic)));
  }


  @Bean
  public Map<String, Object> producerFactory() {
    Map<String, Object> senderConfigProps = new HashMap<>();
    senderConfigProps.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
    senderConfigProps.put(ProducerConfig.ACKS_CONFIG, acks);
    senderConfigProps.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
    senderConfigProps.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, JsonSerializer.class);
    return senderConfigProps;
  }


  @Bean
  public KafkaSender<String, WarehouseOrderRequest> kafkaOMSProducer() {
    SenderOptions<String, WarehouseOrderRequest> senderOptions = SenderOptions.create(producerFactory());
    return KafkaSender.create(senderOptions);
  }

  @Bean
  public KafkaSender<String, WarehouseOrderResponse> kafkaUpdateEventProducer() {
    SenderOptions<String, WarehouseOrderResponse> senderOptions = SenderOptions.create(producerFactory());
    return KafkaSender.create(senderOptions);
  }
}
